'use strict';

var core = require('@mikro-orm/core');
var utils = require('./utils');

var __defProp = Object.defineProperty;
var __name = (target, value) => __defProp(target, "name", { value, configurable: true });
const operatorToReverseOperator = {
  $gt: "$lt",
  $lt: "$gt"
};
const queryOrderToOperator = {
  [core.QueryOrder.ASC]: "$gt",
  [core.QueryOrder.ASC_NULLS_LAST]: "$gt",
  [core.QueryOrder.ASC_NULLS_FIRST]: "$gt",
  [core.QueryOrder.DESC]: "$lt",
  [core.QueryOrder.DESC_NULLS_LAST]: "$lt",
  [core.QueryOrder.DESC_NULLS_FIRST]: "$lt",
  [core.QueryOrder.asc]: "$gt",
  [core.QueryOrder.asc_nulls_last]: "$gt",
  [core.QueryOrder.asc_nulls_first]: "$gt",
  [core.QueryOrder.desc]: "$lt",
  [core.QueryOrder.desc_nulls_last]: "$lt",
  [core.QueryOrder.desc_nulls_first]: "$lt"
};
const queryOrderToReverseQueryOrder = {
  [core.QueryOrder.ASC]: core.QueryOrder.DESC,
  [core.QueryOrder.ASC_NULLS_LAST]: core.QueryOrder.DESC_NULLS_FIRST,
  [core.QueryOrder.ASC_NULLS_FIRST]: core.QueryOrder.DESC_NULLS_LAST,
  [core.QueryOrder.DESC]: core.QueryOrder.ASC,
  [core.QueryOrder.DESC_NULLS_LAST]: core.QueryOrder.ASC_NULLS_FIRST,
  [core.QueryOrder.DESC_NULLS_FIRST]: core.QueryOrder.ASC_NULLS_LAST,
  [core.QueryOrder.asc]: core.QueryOrder.desc,
  [core.QueryOrder.asc_nulls_last]: core.QueryOrder.desc_nulls_first,
  [core.QueryOrder.asc_nulls_first]: core.QueryOrder.desc_nulls_last,
  [core.QueryOrder.desc]: core.QueryOrder.asc,
  [core.QueryOrder.desc_nulls_last]: core.QueryOrder.asc_nulls_first,
  [core.QueryOrder.desc_nulls_first]: core.QueryOrder.asc_nulls_last
};
class Cursor {
  static {
    __name(this, "Cursor");
  }
  state;
  constructor(state) {
    this.state = state;
  }
  static fromEntity(entity, orderBy) {
    if (Array.isArray(orderBy) && orderBy.length === 0 || Object.keys(orderBy).length === 0) {
      throw new Error("Cannot create cursor with empty orderBy");
    }
    const pojo = core.wrap(entity).toPOJO();
    const state = [];
    const orderByArr = Array.isArray(orderBy) ? orderBy : [
      orderBy
    ];
    orderByArr.forEach((order) => {
      const walkQueryOrderMap = /* @__PURE__ */ __name((order2, prevKeys = []) => {
        Object.entries(order2).forEach(([key, value]) => {
          const currKeys = [
            ...prevKeys,
            key
          ];
          if (typeof value === "string") {
            const strKey = currKeys.join(".");
            let data = pojo;
            for (const key2 of currKeys) {
              data = data[key2];
            }
            state.push([
              strKey,
              value,
              data
            ]);
          }
          if (typeof value === "object") {
            walkQueryOrderMap(value, currKeys);
          }
        });
      }, "walkQueryOrderMap");
      walkQueryOrderMap(order);
    });
    return new Cursor(state);
  }
  static fromBase64(base64) {
    const state = JSON.parse(Buffer.from(base64, "base64").toString("utf-8"));
    return new Cursor(state);
  }
  toBase64() {
    return Buffer.from(this.toJSON()).toString("base64");
  }
  toJSON() {
    return JSON.stringify(this.state);
  }
  buildOrderBy({ direction }) {
    return this.state.reduce((acc, curr) => {
      const [key, order] = curr;
      utils.set(acc, key.toString(), direction === "forward" ? order : queryOrderToReverseQueryOrder[order]);
      return acc;
    }, {});
  }
  buildWhereOr({ direction }) {
    return this.state.map(([key, order, value], i) => {
      const filterQuery = {};
      let operator = queryOrderToOperator[order];
      if (direction === "backward") operator = operatorToReverseOperator[operator];
      utils.set(filterQuery, key.toString(), {
        [operator]: value
      });
      this.state.filter((_, j) => j < i).forEach(([key2, _, value2]) => {
        utils.set(filterQuery, key2.toString(), value2);
      });
      return filterQuery;
    });
  }
}

exports.Cursor = Cursor;
