import { __name } from './chunk-O6YSETKJ.mjs';
import { QueryOrder, wrap } from '@mikro-orm/core';
import { set } from './utils';

const operatorToReverseOperator = {
  $gt: "$lt",
  $lt: "$gt"
};
const queryOrderToOperator = {
  [QueryOrder.ASC]: "$gt",
  [QueryOrder.ASC_NULLS_LAST]: "$gt",
  [QueryOrder.ASC_NULLS_FIRST]: "$gt",
  [QueryOrder.DESC]: "$lt",
  [QueryOrder.DESC_NULLS_LAST]: "$lt",
  [QueryOrder.DESC_NULLS_FIRST]: "$lt",
  [QueryOrder.asc]: "$gt",
  [QueryOrder.asc_nulls_last]: "$gt",
  [QueryOrder.asc_nulls_first]: "$gt",
  [QueryOrder.desc]: "$lt",
  [QueryOrder.desc_nulls_last]: "$lt",
  [QueryOrder.desc_nulls_first]: "$lt"
};
const queryOrderToReverseQueryOrder = {
  [QueryOrder.ASC]: QueryOrder.DESC,
  [QueryOrder.ASC_NULLS_LAST]: QueryOrder.DESC_NULLS_FIRST,
  [QueryOrder.ASC_NULLS_FIRST]: QueryOrder.DESC_NULLS_LAST,
  [QueryOrder.DESC]: QueryOrder.ASC,
  [QueryOrder.DESC_NULLS_LAST]: QueryOrder.ASC_NULLS_FIRST,
  [QueryOrder.DESC_NULLS_FIRST]: QueryOrder.ASC_NULLS_LAST,
  [QueryOrder.asc]: QueryOrder.desc,
  [QueryOrder.asc_nulls_last]: QueryOrder.desc_nulls_first,
  [QueryOrder.asc_nulls_first]: QueryOrder.desc_nulls_last,
  [QueryOrder.desc]: QueryOrder.asc,
  [QueryOrder.desc_nulls_last]: QueryOrder.asc_nulls_first,
  [QueryOrder.desc_nulls_first]: QueryOrder.asc_nulls_last
};
class Cursor {
  static {
    __name(this, "Cursor");
  }
  state;
  constructor(state) {
    this.state = state;
  }
  static fromEntity(entity, orderBy) {
    if (Array.isArray(orderBy) && orderBy.length === 0 || Object.keys(orderBy).length === 0) {
      throw new Error("Cannot create cursor with empty orderBy");
    }
    const pojo = wrap(entity).toPOJO();
    const state = [];
    const orderByArr = Array.isArray(orderBy) ? orderBy : [
      orderBy
    ];
    orderByArr.forEach((order) => {
      const walkQueryOrderMap = /* @__PURE__ */ __name((order2, prevKeys = []) => {
        Object.entries(order2).forEach(([key, value]) => {
          const currKeys = [
            ...prevKeys,
            key
          ];
          if (typeof value === "string") {
            const strKey = currKeys.join(".");
            let data = pojo;
            for (const key2 of currKeys) {
              data = data[key2];
            }
            state.push([
              strKey,
              value,
              data
            ]);
          }
          if (typeof value === "object") {
            walkQueryOrderMap(value, currKeys);
          }
        });
      }, "walkQueryOrderMap");
      walkQueryOrderMap(order);
    });
    return new Cursor(state);
  }
  static fromBase64(base64) {
    const state = JSON.parse(Buffer.from(base64, "base64").toString("utf-8"));
    return new Cursor(state);
  }
  toBase64() {
    return Buffer.from(this.toJSON()).toString("base64");
  }
  toJSON() {
    return JSON.stringify(this.state);
  }
  buildOrderBy({ direction }) {
    return this.state.reduce((acc, curr) => {
      const [key, order] = curr;
      set(acc, key.toString(), direction === "forward" ? order : queryOrderToReverseQueryOrder[order]);
      return acc;
    }, {});
  }
  buildWhereOr({ direction }) {
    return this.state.map(([key, order, value], i) => {
      const filterQuery = {};
      let operator = queryOrderToOperator[order];
      if (direction === "backward") operator = operatorToReverseOperator[operator];
      set(filterQuery, key.toString(), {
        [operator]: value
      });
      this.state.filter((_, j) => j < i).forEach(([key2, _, value2]) => {
        set(filterQuery, key2.toString(), value2);
      });
      return filterQuery;
    });
  }
}

export { Cursor };
